<?php
namespace Teknisa\GOP\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\GER\Utilities\Environment;
use Zeedhi\Framework\DTO\Response\Error;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;	
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\GOP\Services\Gop10000CalculoOrcamentoOperacionalService;
use PHPExcel_IOFactory;

class Gop10000CalculoOrcamentoOperacionalController extends \Teknisa\GER\Controllers\TeknisaCrud {
	protected $dataSourceName = "itemorcagop";
	protected $parameterBagColumns;
	protected $connection;
	protected $dataSourceManager;
	protected $novoCodigo;
	protected $environment;
	protected $sheetImporter;
	protected $util;

	public function __construct($crudService, $parameterBag, $dataSourceManager, $connection, $novoCodigo, $environment, Gop10000CalculoOrcamentoOperacionalService $service, $sheetImporter, $util){
		$this->dataSourceManager = $dataSourceManager;
		$this->connection		 = $connection;
		$this->novoCodigo        = $novoCodigo;
		$this->environment       = $environment;
		$this->service  		 = $service;
		$this->sheetImporter	 = $sheetImporter;
		$this->util              = $util;

		parent::__construct($crudService, $parameterBag);
	}
		
	public function atualizaMesBase(DTO\Request $request, DTO\Response $response){
	       
        $dataSet = $request->getDataSet();
        $erroList = array();
        if($dataSet != null){
        	$rows = $dataSet->getRows();
        	
        	if(!empty($rows)){

        		foreach($rows as $key => $row){
	        		try{
		                $parametros = array(
	                        'P_DTMESBASEITEM' => $row["DTMESBASEITEM"],
	                        'P_NRITEMORCAOP' => $row["NRITEMORCAOP"]
		                );
		                $this->service->updateMesBase($parametros);
		                
		            }catch(\Exception $ex){
		                array_push($erroList, "MSG_ERRO" . $ex->getMessage());
		            }
				}
        		
        		if(!empty($erroList)){
	                $response->addDataSet(new DataSet("errorList", $erroList));
	                throw new \Exception ("Ocorreram erros durante a operação. Verifique o log");
	            }
        	}
        }
    }
    public function atualizaItemTotal(DTO\Request $request, DTO\Response $response){
	    
	    $dados = $request->getDataSet();
	    
        if(!empty($dados)){
        	$rows = $dados->getRows();
        	foreach($rows as $row){
        		$nrItemOrcaMes = $row['NRITORCAMESGOP'];
        		$nrItDetalOrca = $row['NRITDETALORCAMESGOP'];
        		$nrOrcamento = $row['VRORCAMENTO'];
        	
    			$params1 =  array(
	                        'NRITORCAMESGOP' => $nrItemOrcaMes,
	                        'NRITDETALORCAMESGOP' => $nrItDetalOrca,
	                        'VRORCAMENTO' => $nrOrcamento
	            );
	            $params2 = array(
	            			'P_NRITORCAMESGOP' => $nrItemOrcaMes
	            );
	            
	            $this->service->updateItem($params1);
	            
	            if(empty($rows[0]['IDALTMANUAL'])){
					$erroList = $this->service->atualizaItemTotal($params2);
	            }
        		
        		if(!empty($erroList)){
	                $response->addDataSet(new DataSet("errorList", $erroList));
	                throw new \Exception ("Ocorreram erros durante a operação. Verifique o log");
	            }
        	}
        }
    }
    public function processaCalculoComensais(DTO\Request $request, DTO\Response $response){
	       
        $dataSet = $request->getDataSet();
        $parametros = $request->getParameter("parametros");
        
        foreach ($parametros as $key => $parametro) {
        	if($parametro["name"] == "P_NRANOORCAGOP")
        		$P_NRANOORCAGOP = $parametro["value"];
        	if($parametro["name"] == "P_IDPERIODOORCA")
        		$P_IDPERIODOORCA = $parametro["value"];
        	if($parametro["name"] == "P_NRPERIODOORCA")
        		$P_NRPERIODOORCA = $parametro["value"];
        	if($parametro["name"] == "P_IDTIPOORCAOP")
        		$P_IDTIPOORCAOP = $parametro["value"];
        	if($parametro["name"] == "P_PROCESSA_PERCENTUAL")
        		$P_PERCENTUAL = $parametro["value"];
        }
        
        if($dataSet != null){
        	
        	$rows = $dataSet->getRows();
        	
        	if(!empty($rows)){
        		
				$nrLista     = $this->novoCodigo ->getCodeByTableName('LISTA', $this->environment->getNrOrgTrab());
				$currentTime = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
				
        		foreach($rows as $key => $row){
	        			$lista                   = array();
			            $lista["NRLISTA"]        = $nrLista;
			            $lista["CDLISTA"]        = $row["CDFILIAL"];
			            $lista["__is_new"]       = true;
			            $lista["DTULTATULIST"]   = "";
			            $lista["DTINCLUSAO"]     = $currentTime;
			            $lista["DTULTATU"]       = "";
			            $lista["NRORGINCLUSAO"]  = $this->environment->getNrOrgTrab();
			            $lista["CDOPERINCLUSAO"] = "";
			            $lista["NRORGULTATU"]    = "";
			            $lista["CDOPERULTATU"]   = "";
			            $lista["IDATIVO"]        = "";
			            $lista["NRORG"]          = $this->environment->getNrOrgTrab();
			            $lista["IDATIVO"]        = 'S';
			            
			            $rowsToSave[] = $lista;
        		}
        		
        		
        		try{
        			$this->crudService->save("lista", $rowsToSave);
        		}catch(\Exception $ex){
        			 $response->setError(new Error("Nao foi possivel processar os dados enviados   --- " . $ex->getMessage(), 500));
        		}
        		
        		try{
	        		$procedure = new StoredProcedure($this->connection, 'CALCULA_ORCAMENTO_GOP');
			        $procedure->addParam(new Param('P_NRANOORCAGOP', Param::PARAM_INPUT, $P_NRANOORCAGOP, Param::PARAM_TYPE_INT));
			        $procedure->addParam(new Param('P_IDPERIODOORCA', Param::PARAM_INPUT, $P_IDPERIODOORCA, Param::PARAM_TYPE_STR));
			        $procedure->addParam(new Param('P_NRPERIODOORCA', Param::PARAM_INPUT, $P_NRPERIODOORCA, Param::PARAM_TYPE_INT));
			        $procedure->addParam(new Param('P_IDTIPOORCAOP', Param::PARAM_INPUT, $P_IDTIPOORCAOP, Param::PARAM_TYPE_STR));
			        
			        $procedure->addParam(new Param('P_PERCENTUAL', Param::PARAM_INPUT, isset($P_PERCENTUAL)?$P_PERCENTUAL:"N", Param::PARAM_TYPE_STR));
			        $procedure->addParam(new Param('P_LISTAFIL', Param::PARAM_INPUT, $nrLista, Param::PARAM_TYPE_STR));
			        $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $this->environment->getCdOperador(), Param::PARAM_TYPE_STR)); 
			        $procedure->addParam(new Param('P_RETORNO', Param::PARAM_OUTPUT,null,Param::PARAM_TYPE_STR));
			        $procedure->execute();
			    
			       	try{
			    		$filtro = array(
                		array(
                    		"columnName"=> "NRLISTA",
                    		"operator"	=> "=",
                    		"value" 	=> $nrLista
                			)
            			);
            			
						$dataSet = $this->crudService->getFilteredDataSet('lista', $filtro);
						$this->crudService->delete($response, $dataSet);
			    		
			    	}catch(\Exception $except){
			    		 $response->setError(new Error("Erro ao excluir lista de orçamento :" . $except->getMessage(), 500));	
			    	}
			    
				}catch(\Exception $ex){
        			 $response->setError(new Error("Nao foi possivel processar os dados enviados   --- " . $ex->getMessage(), 500));
        		}
        	}
        }
    }
	
	public function listaAno (DTO\Request $request, DTO\Response $response){
		
		$conditions = $request->getFilterCriteria()->getConditions();
		$ano = intval(date('Y'))- 20;
		$anoFinal = $ano + 25;
		
		for($ano; $ano < $anoFinal; $ano++){
			$dados = new Ano($ano);
			$dataSet[] = $dados;
		}
		
		if(isset($conditions[0]['value'])){
			
			$busca = str_replace("%","",$conditions[0]['value']);
			if(!empty($busca)){
				$newDataSet = "";
				$new_array = array_filter($dataSet, function($obj) use($busca,&$newDataSet){
				    if(stristr($obj->ANO, $busca)){
				    	$newDataSet[] = $obj;
				    }
				    return (stristr($obj->ANO, $busca));
				});
				$dataSet = $newDataSet;
			}
		}
		$response->addDataSet(new DataSet('data', $dataSet));
	}
	
	public function updateMeses(DTO\Request $request, DTO\Response $response){

       $dados = $request->getDataSet();
       
       if(!empty($dados)){
           $rows = $dados->getRows();
			$a = [];
           if(!empty($rows)){
           		
               foreach($rows as $row){
               		foreach($row['changedFields'] as $mes){
               			$params =  array(
                            ':P_NRITEMORCAOP'  => $row['NRITEMORCAOP'],
                            ':P_NRANOORCAGOP'  => $row['NRANOORCAGOP'],
                            ':P_IDPERIODOORCA' => $row['IDPERIODOORCA'],
                            ':P_NRPERIODOORCA' => $row['NRPERIODOORCA'],
                            ':P_MES' => $mes,
                            'P_CDFILIAL' => $row['CDFILIAL']
                        );
                    	$retorno   = $this->service->findNritorcamesgop($params);
                        $stringMes = $this->service->getMes($mes);
                    
                        $parametros =  array(
                        	'VRORCAMENTO'    => $row[$stringMes],
                            'NRITORCAMESGOP' => $retorno[0]['NRITORCAMESGOP'],
                            'NRITEMORCAOP'   => $row['NRITEMORCAOP']
                        );
                        try{
                        	$this->service->updateMeses($parametros);
                        
                        }catch(\Exception $ex){
                            $response->setError(new Error("Nao foi possivel processar os dados enviados   --- " . $ex->getMessage(), 500));
                        }
               		}
               }
            } 
        }
    }
    
    public function updateDetalheImposto(DTO\Request $request, DTO\Response $response){
    	
    	$dados = $request->getDataSet();
    	
    	if(!empty($dados)){
    		
           $rows = $dados->getRows();
           
           if(!empty($rows)){
                try{
                	$this->service->updateDetalheImposto($rows);
                }catch(\Exception $ex){
                    $response->setError(new Error("Nao foi possivel processar os dados enviados   --- " . $ex->getMessage(), 500));
                }
            }
        } 
    }
    
    
    public function updateDetalheOrcaMo(DTO\Request $request, DTO\Response $response){
    
    	$dados = $request->getDataSet();
    	
    	if(!empty($dados)){
    		
           $rows = $dados->getRows();
           
           if(!empty($rows)){
                try{
                	$this->service->updateDetalheOrcaMo($rows);
                }catch(\Exception $ex){
                    $response->setError(new Error("Nao foi possivel processar os dados enviados   --- " . $ex->getMessage(), 500));
                }
            }
        } 	
   
    }
    
    
   
    public function updateTotalImposto(DTO\Request $request, DTO\Response $response){
    	
    	$dados = $request->getDataSet();
    	if(!empty($dados)){
           $row = $dados->getRows();
           if(!empty($row)){
               $parametros = array(
                    ":VRTOTALMES" => $row[0]["VRTOTALMES"],
                    ":P_MES"      => $row[0]["MES"]
                );
                try{
                	$this->service->updateTotalImposto($parametros, $row[0]["nritemorcaop"]);
                }catch(\Exception $ex){
                    $response->setError(new Error("Nao foi possivel processar os dados enviados   --- " . $ex->getMessage(), 500));
                }
            } 
        }
    }
    
    public function updateDetalheDespesaMo(DTO\Request $request, DTO\Response $response){
    	
    	$rows = $request->getDataSet()->getRows();
           if(!empty($rows)){
            try{
               foreach($rows as $row){
               	
                	$parametros = array(
                        "VRORCAMENTO"    => $row["VRORCAMENTO"],
						"NRITDETALORCAMESGOP" => $row["NRITDETALORCAMESGOP"]
                    );

                  $this->service->updateDetalheDespesaMo($parametros);
                }
                $param = array(
            		"VRORCAMENTOMES" => $rows[0]["VRORCAMENTOMES"],
					"NRITORCAMESGOP" => $rows[0]["NRITORCAMESGOP"]
            	);
                
                $this->service->updateDetalheMesDespesaMO($param);
           	}catch(\Exception $ex){
               $response->setError(new Error("Nao foi possivel processar os dados enviados   --- " . $ex->getMessage(), 500));
        	}
                
            } 
    }
    
    public function updateTotalDetalhe(DTO\Request $request, DTO\Response $response){
    	$dados = $request->getDataSet();
    	
    	if(!empty($dados)){
    	    
           $rows = $dados->getRows();

           if(!empty($rows)){
                try{
                	$this->service->updateTotalDetalhe( $rows);
                }catch(\Exception $ex){
                    $response->setError(new Error("Nao foi possivel processar os dados enviados   --- " . $ex->getMessage(), 500));
                }
            } 
        }
    }
    
    public function getNranoorcagop(DTO\Request $request, DTO\Response $response){
    	try{
        	$this->service->getNranoorcagop( $rows);
        }catch(\Exception $ex){
            $response->setError(new Error("Nao foi possivel processar os dados enviados   --- " . $ex->getMessage(), 500));
        }
    }
    
    public function importSpreadsheet(DTO\Request $request, DTO\Response $response) {
		try {
			$this->service->beginTransaction();
			
			$row = $request->getRow();
			$paramOrca = $this->service->getParamOrca();
			
			$sheetPath = $this->sheetImporter->createTemporaryFile($row["SPREADSHEET"]);
			$worksheet = $this->sheetImporter->loadSheet($sheetPath);
			$sheet = $this->sheetImporter->parseSheet($worksheet);
			
			$this->sheetImporter->validateSheetColumns($sheet);
			$this->sheetImporter->validateSheetRows($sheet, $response);
			
			$dataArrays = $this->sheetImporter->validateAndInsertData($row, $sheet, $response, $paramOrca);
			$this->sheetImporter->startImportProcedure();
			
			$this->service->commit();
			
			$errors = $this->sheetImporter-> checkForErrors($dataArrays);
			
			if (isset($errors[0])) {
		        $response->addDataSet(new DataSet("errors", $errors));
		    }
		} catch (\Exception $e) {
			$this->service->rollback();
			$response->setError(new Error($e->getMessage(), 500));
		} finally {
			if (isset($sheetPath)) {
                $this->sheetImporter->removeTemporaryFile($sheetPath);
            }
		}
    }
    
    public function unidadeDownloadPlanilha(DTO\Request $request, DTO\Response $response) {
		$idSisDos = $this->service->getParamgeral()[0]["IDSISDOS"];
    	$cdOperador = $this->environment->getCdoperador();
    	
    	if ($idSisDos == 3) {
    		$this->dataSourceName = "gop10000_unidadeDownloadPlanilha_GR";
    	} else {
    		$this->dataSourceName = "gop10000_unidadeDownloadPlanilha";
    	}
    	
    	$filterCriteria = $request->getFilterCriteria();
    	$filterCriteria->addCondition("P_CDOPERADOR", $cdOperador);
    	
		$this->find($request, $response);
    }
    
    public function confirmContabil(DTO\Request $request, DTO\Response $response) {
    	$row = $request->getRow();
    	
    	try{
    		$conditions = array(
    			array(
                    "columnName"=>"P_NRORG", 
                    "operator"=>"=", 
                    "value"=>$this->environment->getNrOrg()
                )
    		);
    		
    		if($row['ROWS']['P_TODOS'] == 'S'){
    			$results = $this->crudService->getFilteredDataSet('oc_contabil_cent_cust', $conditions);
    			$dados = $results->getRows();
    			$cdCentCust = array();

    			foreach($dados as $result) {
    				array_push($cdCentCust, $result['CDCENTCUST']);
    			}
    			
    			$nrLista = $this->util->insertLista($cdCentCust);
    			
    		}else {
    			$nrLista = $this->util->insertLista($row['ROWS']['CDCENTCUST']);
    		}
    		
    		$filial = array();
   
    		foreach($row['ROWSCHECKED'] as $cdFilial) {
    			array_push($filial, $cdFilial['CDFILIAL']);
    		}
    		
    		$rowsChecked = $row['ROWSCHECKED'][0]; 
    		$rows = $row['ROWS'];
    		
    		$nrListaFilial = $this->util->insertLista($filial);
       		$this->service->confirmContabil($rows, $rowsChecked, $nrLista, $nrListaFilial);
			$this->util->deleteLista($nrLista);
			$this->util->deleteLista($nrListaFilial);
    	}catch(\Exception $e){
    		$response->setError(new Error($e->getMessage(), 500));
    	} 
    }
}

class Ano {
    public $ANO;
    function __construct($ano = "") {
        $this->ANO = (string)$ano;
    }
    
}