<?php
namespace Teknisa\GOP\Modules\Parametro\Controllers; 

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\Controller;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO\Response\Notification;



class Gop00200ControleUnidadeGerenteController extends \Teknisa\GER\Controllers\TeknisaCrud {
    
    protected $crudService;
    protected $connection;
    
    public function __construct($crudService, ParameterBag $parameterBag, $dataSourceManager, $connection) {
        $this->dataSourceManager    = $dataSourceManager;
        $this->connection    		= $connection;
        parent::__construct($crudService, $parameterBag);
    }
    

    public function excluirGrupoGerente(DTO\Request $request, DTO\Response $response ){
        $rows = $request->getDataSet()->getRows();

        foreach($rows as $row){
            try{
                
                $this->connection->beginTransaction();
                
                $conditions = array (
        			array (
            				"columnName"    =>"CDGRUPGERENTEOP",
            				"operator"      =>"=",
            				"value"         => $row["CDGRUPGERENTEOP"]
            			)
        		);
        		
        		$operadores = $this->crudService->getFilteredDataSet('opgrupgerenteop', $conditions)->getRows();

        		if(!empty($operadores)){
        		    $menssagem = array();
        		    array_push($menssagem, "Não é possível excluir grupo com gerentes cadastrados!");
        		    $response->addDataSet(new DataSet("errorList", $menssagem));
        		    return 0;
        		}else if(empty($operadores)){

            		$filiais = $this->crudService->getFilteredDataSet('filialoperop', $conditions);
            		$this->dataSourceManager->delete($filiais);
            		
            		$grupo = $this->crudService->getFilteredDataSet('grupgerenteop', $conditions);
            		$this->dataSourceManager->delete($grupo);
            		
            		$this->connection->commit();
        		}

        		
            } catch (\Exception $e) {
    			$this->connection->rollback();
                $response->setError(new DTO\Response\Error($e->getMessage(), 1));
            }
            
        }
    }
    
    public function saveOperadorGrupo(DTO\Request $request, DTO\Response $response ){
        $row = $request->getDataSet()->getRows()[0];
        
        try{
            if(empty($row))
                throw new \Exception("Nao foi possivel realizar a operação!");
                
            $validaOperador = $this->validaOperadorGrupo($row["CDOPERADOR"]);
            
            if(!empty($validaOperador)){
                if ($row['CDGRUPGERENTEOP'] == $validaOperador[0]['CDGRUPGERENTEOP']){
                    throw new \Exception("Operador vinculado ao grupo atual! ");
                }else {
                    throw new \Exception("Operador vinculado em outro grupo!");
                }
            }
                
            $newCode = array(
                            'CDOPERADOR' => $row["CDOPERADOR"],
                            'CDGRUPGERENTEOP' => $row["CDGRUPGERENTEOP"],
                            '__is_new' => true
                        );
            
            $rowInsert = new DTO\Request\Row($newCode, '', '', '');
           
            $rowToSave =    $this->crudService->getRowsToSave($rowInsert, "opgrupgerenteop"); 
            $this->crudService->save("opgrupgerenteop", $rowToSave);
            
        } catch (\Exception $e) {
            
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }
    
    public function excluirOperadorGrupo(DTO\Request $request, DTO\Response $response ){
        $rows = $request->getDataSet()->getRows();

        foreach($rows as $row){
            try{
                
                $this->connection->beginTransaction();
                
                $conditions = array (
        			array (
            				"columnName"    =>"CDGRUPGERENTEOP",
            				"operator"      =>"=",
            				"value"         => $row["CDGRUPGERENTEOP"]
            			),
            		array (
            				"columnName"    =>"CDOPERADOR",
            				"operator"      =>"=",
            				"value"         => $row["CDOPERADOR"]
            			)
        		);
        		
        		$operadores = $this->crudService->getFilteredDataSet('opgrupgerenteop', $conditions);
                $this->dataSourceManager->delete($operadores);
        		
        		$filiais = $this->crudService->getFilteredDataSet('filialoperop', $conditions);
        		$this->dataSourceManager->delete($filiais);

        		$this->connection->commit();
        		

            } catch (\Exception $e) {
    			$this->connection->rollback();
                $response->setError(new DTO\Response\Error($e->getMessage(), 1));
            }
            
        }
    }
    
    public function validaOperadorGrupo($cdOperador){
        $conditions = array (
        			array (
            				"columnName"    =>"CDOPERADOR",
            				"operator"      =>"=",
            				"value"         => $cdOperador
            			)
        		);
		
		$dataSet = $this->crudService->getFilteredDataSet('opgrupgerenteop', $conditions);
		
		return  $dataSet->getRows();
		
    }
    
}

